Package: EBImage
Version: 4.54.0
Title: Image processing and analysis toolbox for R
Encoding: UTF-8
Author: Andrzej Oleś, Gregoire Pau, Mike Smith, Oleg Sklyar, Wolfgang
        Huber, with contributions from Joseph Barry and Philip A.
        Marais
Maintainer: Andrzej Oleś <andrzej.oles@gmail.com>
Depends: methods
Imports: BiocGenerics (>= 0.7.1), graphics, grDevices, stats, abind,
        tiff, jpeg, png, locfit, fftwtools (>= 0.9-7), utils,
        htmltools, htmlwidgets, RCurl
Suggests: BiocStyle, digest, knitr, rmarkdown, shiny
Description: EBImage provides general purpose functionality for image
        processing and analysis. In the context of (high-throughput)
        microscopy-based cellular assays, EBImage offers tools to
        segment cells and extract quantitative cellular descriptors.
        This allows the automation of such tasks using the R
        programming language and facilitates the use of other tools in
        the R environment for signal processing, statistical modeling,
        machine learning and visualization with image data.
License: LGPL
LazyLoad: true
biocViews: Visualization
VignetteBuilder: knitr
URL: https://github.com/aoles/EBImage
BugReports: https://github.com/aoles/EBImage/issues
Config/pak/sysreqs: cmake libfftw3-dev make libjpeg-dev libpng-dev
        libtiff-dev libuv1-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:31:40 UTC
RemoteUrl: https://github.com/bioc/EBImage
RemoteRef: RELEASE_3_23
RemoteSha: 183e0037ac81df55139b8e9a569b95719ea96c9b
NeedsCompilation: yes
Packaged: 2026-04-29 13:00:36 UTC; root
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 13:05:39 UTC; unix
