## Installation

The package is distributed as part of the [Bioconductor](http://bioconductor.org/packages/EBImage) project.
To install this package, start R and enter:

```
install.packages("BiocManager")
BiocManager::install("EBImage")
```

## Documentation

For an overview of the package functionality, see the [package vignette](http://bioconductor.org/packages/devel/bioc/vignettes/EBImage/inst/doc/EBImage-introduction.html).

Once you install the package, you can access individual man pages by a call to, e.g., `?Image`.

## FAQ

See [my answers on Stack Overflow](http://stackoverflow.com/search?q=user%3A2792099+%09is%3Aanswer+EBimage).

## Bug reports and feature requests

Bug reports, feature requests, or any other issues with the package should be reported at https://github.com/aoles/EBImage/issues.
